import 'package:apply/apply.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/elements/attributes.dart';
import 'package:apply/handler/animation_handler.dart';
import 'package:apply/handler/box_handler.dart';
import 'package:apply/handler/display_handler.dart';
import 'package:apply/handler/transform_handler.dart';
import 'package:apply/responsive/responsive_service.dart';
import 'package:flutter/material.dart';

// ignore: must_be_immutable
class Div extends StatelessWidget {
  final List<Widget> children;
  Set<Plugin>? plugins;
  Set<Plugin>? ownPlugins;
  AnimationConfig animationConfig = AnimationConfig();
  DivSpacing? spacing;

  final String style;
  Attributes? attributes;
  Pair<double, double>? widthAndHeightPair;
  final DivOptions divOptions;

  Div._(this.children, this.style, this.divOptions);

  Div({
    String style = "",
    List<Widget> children = const [],
    Widget? child,
    Color? color,
    double? height,
    double? width,
    EdgeInsets? margin,
    EdgeInsets? padding,
    BoxDecoration? decoration,
    VoidCallback? onTap,
  })  : this.children = child != null ? [child] : children,
        this.style = style,
        this.divOptions = DivOptions(
          color: color,
          decoration: decoration,
          height: height,
          width: width,
          margin: margin,
          padding: padding,
          onTap: onTap,
        );

  @override
  Widget build(BuildContext context) {
    setPlugins(context);

    ComposableWidget start = (wb) => Builder(
          builder: (context) => wb(context),
        );
    var displayHandler = DisplayHandler(this, context);
    var boxHandler = BoxHandler(this, start)();
    var transformHandler = TransformHandler(this, boxHandler)();
    var animationHandler = AnimationHandler(this, transformHandler, context)();

    WidgetBuilder displayWb = displayHandler();
    Widget boxModeledWidget = animationHandler(displayWb);

    var finalWidget = displayHandler.addMargin(boxModeledWidget);
    return finalWidget(context);
  }

  void setPlugins(BuildContext context) {
    setAttributes(context);

    var enclosingDiv = context.findAncestorWidgetOfExactType<Div>();

    if (enclosingDiv != null) {
      var inheritedPlugins = enclosingDiv.plugins?.getInheritedPlugins();
      plugins = attributes?.getAppliedPlugins(inheritedPlugins ?? []);
    } else {
      plugins = attributes?.getAppliedPlugins();
    }
  }

  void setAttributes(BuildContext context) {
    var parts = style.split(" ");
    var size = ApplyMediaQuery.size ?? MediaQuery.of(context).size;
    var filteredAttributes = ResponsiveService.instance.filterBreakPoints(
      parts,
      size,
    );

    attributes = Attributes.fromParts(filteredAttributes);
    // print(["Size je: ", attributes]);
  }

  void setSpacing(double? a, double? b, bool first, bool last) => spacing = DivSpacing(a, b, first, last);
}

class DivOptions {
  final Color? color;
  final BoxDecoration? decoration;
  final double? width;
  final double? height;
  final EdgeInsets? margin;
  final EdgeInsets? padding;
  final AlignmentGeometry? alignment;
  final VoidCallback? onTap;

  const DivOptions({
    this.color,
    this.decoration,
    this.width,
    this.height,
    this.margin,
    this.padding,
    this.alignment,
    this.onTap,
  });
}

class AnimationConfig {
  int animationIndex;
  int duration;
  int delay;

  AnimationConfig({
    this.animationIndex = 0,
    this.duration = 3000,
    this.delay = 0,
  });
}

class DivSpacing{
  final double? spaceX;
  final double? spaceY;
  final bool isFirst;
  final bool isLast;

  DivSpacing(this.spaceX, this.spaceY, this.isFirst, this.isLast);

  @override
  String toString() {
    return '_Spacing{spaceX: $spaceX, spaceY: $spaceY, isFirst: $isFirst, isLast: $isLast}';
  }
}
