import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';
import 'package:flutter/material.dart';

class Space extends Plugin {
  final String id = "space";
  final List<Range> ranges = [
    Range(new RegExp(r"^space-\d{1,3}$"), 1),
    Range(new RegExp(r"^space-x-\d{1,3}$"), 3),
    Range(new RegExp(r"^space-y-\d{1,3}$"), 13),
  ];

  Space([String? value]) {
    setConfig();
    addValue(value);
  }

  void handleFlex(Div element, BuildContext context) {
    _addSpacing(element, context);
  }

  void _addSpacing(Div element, BuildContext context) {
    double? width = values["space|space-x"]?.asDouble() ?? null;
    double? height = values["space|space-y"]?.asDouble() ?? null;
    var children = element.children;
    var length = children.length;
    for (var i = 0; i < length; i++) {
      var widget = children[i];
      var isFirst = i == 0;
      var isLast = i == (length - 1);
      if (length > 1) {
        if (widget is Flexible) {
        } else {
          var nextIsFlexible = false;
          var prevIsFlexible = false;
          if (i + 1 < length - 1) {
            nextIsFlexible = children[i + 1] is Flexible;
          }
          if (i > 0) {
            prevIsFlexible = children[i - 1] is Flexible;
          }
          // var spacing = DivSpacing(width, height, isFirst, isLast);
          if(widget is Div){
            widget.setSpacing(width, height, isFirst, isLast);
          }
          // var edgeInsets = EdgeInsets.only(
          //   left: spacing.isFirst ? 0 : (spacing.spaceX ?? 0) / (prevIsFlexible ? 1 : 2),
          //   right: spacing.isLast ? 0 : (spacing.spaceX ?? 0) / (nextIsFlexible ? 1 : 2),
          //   top: spacing.isFirst ? 0 : (spacing.spaceY ?? 0) / (prevIsFlexible ? 1 : 2),
          //   bottom: spacing.isLast ? 0 : (spacing.spaceY ?? 0) / (nextIsFlexible ? 1 : 2),
          // );

          // children[i] = Padding(
          //   padding: edgeInsets,
          //   child: widget,
          // );
        }
      }
    }
  }

  Pair<double, double> xYPair() =>
      Pair(values["space|space-x"]?.asDouble() ?? 0, values["space|space-y"]?.asDouble() ?? 0);

  @override
  Space createInstance(ElementClass elementClass) => Space(elementClass.value);
}
